@@ -19,11 +19,20 @@
     Boston, MA 02111-1307, USA.
 
     Author: Ettore Perazzoli <ettore@helixcode.com>
+
 */
 
 #include <config.h>
 #include <gnome.h>
 #include <bonobo.h>
+#include <stdio.h>
+#include <glib.h>
+#include <errno.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+
+#include "HTMLEditor.h"
 
 #include "gtkhtml.h"
 #include "htmlengine-edit.h"
@@ -38,8 +47,10 @@
 #include "paragraph.h"
 #include "link.h"
 #include "body.h"
+#include "spell.h"
 
 #include "editor-control-factory.h"
+#include "gtkhtmldebug.h"
 
 #ifdef USING_OAF
 #define CONTROL_FACTORY_ID "OAFIID:control-factory:html-editor:97cebd11-9e8c-45c3-9a0a-5a269c760c00"
@@ -163,6 +174,101 @@ release (GtkWidget *widget, GdkEventButton *event, GtkHTMLControlData *cd)
 	return FALSE;
 }
 
+static int
+load_from_file (GtkHTML *html,
+		const char *url,
+		GtkHTMLStream *handle)
+{
+	unsigned char buffer[4096];
+	int len;
+	int fd;
+        const char *path;
+
+        if (strncmp (url, "file:", 5) != 0) {
+		g_warning ("Unsupported image url: %s", url);
+		return FALSE;
+	} 
+	path = url + 5; 
+
+	if ((fd = open (path, O_RDONLY)) == -1) {
+		g_warning ("%s", g_strerror (errno));
+		return FALSE;
+	}
+      
+       	while ((len = read (fd, buffer, 4096)) > 0) {
+		gtk_html_write (html, handle, buffer, len);
+	}
+
+	if (len < 0) {
+		/* check to see if we stopped because of an error */
+		gtk_html_end (html, handle, GTK_HTML_STREAM_ERROR);
+		g_warning ("%s", g_strerror (errno));
+		return FALSE;
+	}	
+	/* done with no errors */
+	gtk_html_end (html, handle, GTK_HTML_STREAM_OK);
+	close (fd);
+	return TRUE;
+}
+
+static int
+load_from_corba (BonoboControl *control,
+		 const char *url, 
+		 GtkHTMLStream *handle)
+{
+	HTMLEditor_Resolver resolver;
+	Bonobo_UIHandler remote_uih;
+	BonoboUIHandler *uih;
+	CORBA_Environment ev;	
+
+	CORBA_exception_init (&ev);
+	
+	uih = gtk_type_new (bonobo_ui_handler_get_type());
+	remote_uih = bonobo_control_get_remote_ui_handler (control);
+	uih = BONOBO_UI_HANDLER (bonobo_object_construct (BONOBO_OBJECT(uih), remote_uih));
+	if (remote_uih != CORBA_OBJECT_NIL) {
+		resolver = bonobo_object_query_interface (BONOBO_OBJECT (uih), "IDL:HTMLEditor/Resolver:1.0");
+		
+		if (resolver == CORBA_OBJECT_NIL) {
+			/* g_warning ("Unable to aquire resolver interface"); */
+		} else {
+			/* g_warning ("found resolver - rejoice the masses"); */
+			HTMLEditor_Resolver_loadURL (resolver, CORBA_OBJECT_NIL, url, &ev);
+			if (ev._major != CORBA_NO_EXCEPTION){
+				/* g_warning ("Got exception!!!"); */
+			} else {
+				/* g_warning ("No Exceptions made"); */
+			}		
+			
+		}
+	}
+	CORBA_exception_free (&ev);
+
+	return FALSE;
+}
+
+static void
+url_requested_cb (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpointer data)
+{
+	BonoboControl *control;
+
+	g_return_if_fail (data != NULL);
+	g_return_if_fail (url != NULL);
+	g_return_if_fail (handle != NULL);
+
+	control = BONOBO_CONTROL (data);
+
+	if (load_from_corba (control, url, handle))
+		;
+		/* g_warning ("valid corba reponse"); */
+	else if (load_from_file (html, url, handle))
+		;
+		/* g_warning ("valid local reponse"); */
+	else
+		g_warning ("unable to resolve url: %s", url);
+
+}
+
 static gint
 html_button_pressed (GtkWidget *html, GdkEventButton *event, GtkHTMLControlData *cd)
 {
@@ -234,9 +340,16 @@ editor_control_factory (BonoboGenericFactory *factory,
 	gtk_signal_connect (GTK_OBJECT (control), "destroy",
 			    GTK_SIGNAL_FUNC (destroy_control_data_cb), control_data);
 
+	gtk_signal_connect (GTK_OBJECT (html_widget), "url_requested",
+			    GTK_SIGNAL_FUNC (url_requested_cb), control);
+
 	gtk_signal_connect (GTK_OBJECT (html_widget), "button_press_event",
 			    GTK_SIGNAL_FUNC (html_button_pressed), control_data);
 
+#ifdef GTKHTML_HAVE_PSPELL
+	gtk_signal_connect (GTK_OBJECT (html_widget), "spell_suggestion_request",
+			    GTK_SIGNAL_FUNC (spell_suggestion_request_cb), control_data);
+#endif
 	return BONOBO_OBJECT (control);
 }
 
