@@ -46,9 +46,9 @@ load (BonoboPersistStream *ps,
       CORBA_Environment *ev)
 {
 	GtkHTML *html;
-	CORBA_long bytes_read;
 	Bonobo_Stream_iobuf *buffer;
 	GtkHTMLStream *handle;
+	gboolean was_editable;
 
 	if (strcmp (type, "text/html") != 0) {
 		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
@@ -58,25 +58,34 @@ load (BonoboPersistStream *ps,
 
 	html = GTK_HTML (data);
 
+	was_editable = gtk_html_get_editable (html);
+	if (was_editable)
+		gtk_html_set_editable (html, FALSE);
+
 	handle = gtk_html_begin (html);
 
 	do {
-		bytes_read = Bonobo_Stream_read (stream, READ_CHUNK_SIZE,
-						 &buffer, ev);
-		if (ev->_major != CORBA_NO_EXCEPTION) {
-			bytes_read = -1;
+		Bonobo_Stream_read (stream, READ_CHUNK_SIZE,
+				    &buffer, ev);
+		if (ev->_major != CORBA_NO_EXCEPTION)
 			break;
-		}
 
 		gtk_html_write (html, handle, buffer->_buffer, buffer->_length);
 
+		if (buffer->_length <= 0)
+			break;
 		CORBA_free (buffer);
-	} while (bytes_read > 0);
+	} while (1);
 
-	if (bytes_read < 0)
+	if (ev->_major != CORBA_NO_EXCEPTION)
 		gtk_html_end (html, handle, GTK_HTML_STREAM_ERROR);
-	else
+	else {
+		CORBA_free (buffer);
 		gtk_html_end (html, handle, GTK_HTML_STREAM_OK);
+	}
+
+	if (was_editable)
+		gtk_html_set_editable (html, TRUE);
 }
 
 
@@ -160,5 +169,3 @@ persist_stream_impl_new (GtkHTML *html)
 	return bonobo_persist_stream_new (load, save, NULL, get_content_types,
 					  html);
 }
-
-
