@@ -120,6 +120,17 @@ prop_dialog (GtkWidget *mi, GtkHTMLControlData *cd)
 		gtk_html_edit_properties_dialog_set_page (cd->properties_dialog, t);
 }
 
+#ifdef GTKHTML_HAVE_PSPELL
+static void
+spell_suggest (GtkWidget *mi, GtkHTMLControlData *cd)
+{
+	HTMLEngine *e = cd->html->engine;
+
+	gtk_signal_emit_by_name (GTK_OBJECT (cd->html), "spell_suggestion_request",
+				 e->spell_checker, html_engine_get_word (e));
+}
+#endif
+
 #define ADD_ITEM(l,f,t) \
 		menuitem = gtk_menu_item_new_with_label (_(l)); \
                 gtk_object_set_data (GTK_OBJECT (menuitem), "type", GINT_TO_POINTER (t)); \
@@ -152,12 +163,18 @@ popup_show (GtkHTMLControlData *cd, GdkEventButton *event)
 	obj  = cd->html->engine->cursor->object;
 	menu = gtk_menu_new ();
 
+#ifdef GTKHTML_HAVE_PSPELL
+	if (!e->active_selection && obj && html_object_is_text (obj) && !html_engine_word_is_valid (e)) {
+		ADD_SEP;
+		ADD_ITEM ("Suggest word", spell_suggest, -1);
+	}
+#endif
 	if (e->active_selection
 	    || (obj
 		&& (HTML_OBJECT_TYPE (obj) == HTML_TYPE_LINKTEXTMASTER
 		    || (HTML_OBJECT_TYPE (obj) == HTML_TYPE_IMAGE
-			&& (*HTML_IMAGE (obj)->url
-			    || *HTML_IMAGE (obj)->target))))) {
+			&& (HTML_IMAGE (obj)->url
+			    || HTML_IMAGE (obj)->target))))) {
 		    ADD_ITEM (_("Remove link"), remove_link, -1);
 	}
 
@@ -180,8 +197,7 @@ popup_show (GtkHTMLControlData *cd, GdkEventButton *event)
 		ADD_PROP (PARAGRAPH);
 		ADD_ITEM ("Link...", prop_dialog, GTK_HTML_EDIT_PROPERTY_LINK);
 		ADD_PROP (LINK);
-	}
-	if (obj && !e->active_selection) {
+	} else if (obj) {
 		switch (HTML_OBJECT_TYPE (obj)) {
 		case HTML_TYPE_RULE:
 			break;
