@@ -76,6 +76,10 @@ enum {
 	SCROLL,
 	CURSOR_MOVE,
 	INSERT_PARAGRAPH,
+#ifdef GTKHTML_HAVE_PSPELL
+	/* spell checking */
+	SPELL_SUGGESTION_REQUEST,
+#endif
 	/* now only last signal */
 	LAST_SIGNAL
 };
@@ -85,6 +89,7 @@ static guint signals [LAST_SIGNAL] = { 0 };
 static void scroll              (GtkHTML *html, GtkOrientation orientation, GtkScrollType scroll_type, gfloat position);
 static void cursor_move         (GtkHTML *html, GtkDirectionType dir_type, GtkHTMLCursorSkipType skip);
 static void command             (GtkHTML *html, GtkHTMLCommandType com_type);
+static gint mouse_change_pos    (GtkWidget *widget, gint x, gint y);
 
 static void load_keybindings    (GtkHTMLClass *klass);
 
@@ -379,12 +384,24 @@ html_engine_object_requested_cb (HTMLEngine *engine,
 
 /* GtkAdjustment handling.  */
 
+static void
+scroll_update_mouse (GtkWidget *widget)
+{
+	gint x, y;
+
+	if (GTK_WIDGET_REALIZED (widget)) {
+		gdk_window_get_pointer (GTK_LAYOUT (widget)->bin_window, &x, &y, NULL);
+		mouse_change_pos (widget, x, y);
+	}
+}
+
 static void
 vertical_scroll_cb (GtkAdjustment *adjustment, gpointer data)
 {
 	GtkHTML *html = GTK_HTML (data);
 
 	html->engine->y_offset = (gint)adjustment->value;
+	scroll_update_mouse (GTK_WIDGET (data));
 }
 
 static void
@@ -393,6 +410,7 @@ horizontal_scroll_cb (GtkAdjustment *adjustment, gpointer data)
 	GtkHTML *html = GTK_HTML (data);
 		
 	html->engine->x_offset = (gint)adjustment->value;
+	scroll_update_mouse (GTK_WIDGET (data));
 }
 
 static void
@@ -520,7 +538,9 @@ setup_scroll_timeout (GtkHTML *html)
 	html->scroll_timeout_id = gtk_timeout_add (SCROLL_TIMEOUT_INTERVAL,
 						   scroll_timeout_cb, html);
 
+	GDK_THREADS_LEAVE();
 	scroll_timeout_cb (html);
+	GDK_THREADS_ENTER();
 }
 
 static void
@@ -649,7 +669,7 @@ unrealize (GtkWidget *widget)
 {
 	GtkHTML *html = GTK_HTML (widget);
 	
-	html_gdk_painter_unrealize (HTML_GDK_PAINTER (html->engine->painter));
+	html_engine_unrealize (html->engine);
 
 	if (GTK_WIDGET_CLASS (parent_class)->unrealize)
 		(* GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
@@ -709,36 +729,55 @@ size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 	}
 }
 
+static void
+on_url (GtkWidget *widget, HTMLObject *obj)
+{
+	GtkHTML *html = GTK_HTML (widget);
+	const gchar *url;
+
+	g_return_if_fail (obj);
+
+	if ((url = (obj) ? html_object_get_url (obj) : NULL)) {
+		if (html->pointer_url == NULL || strcmp (html->pointer_url, url) != 0) {
+			g_free (html->pointer_url);
+			html->pointer_url = g_strdup (url);
+			gtk_signal_emit (GTK_OBJECT (html), signals[ON_URL], url);
+		}
+
+		if (html->engine->editable)
+			gdk_window_set_cursor (widget->window, html->ibeam_cursor);
+		else
+			gdk_window_set_cursor (widget->window, html->hand_cursor);
+	} else {
+		if (html->pointer_url != NULL) {
+			g_free (html->pointer_url);
+			html->pointer_url = NULL;
+			gtk_signal_emit (GTK_OBJECT (html), signals [ON_URL], NULL);
+		}
+
+		if (obj != NULL && html_object_is_text (obj) && html->allow_selection)
+			gdk_window_set_cursor (widget->window, html->ibeam_cursor);
+		else
+			gdk_window_set_cursor (widget->window, html->arrow_cursor);
+	}
+}
+
 static gint
-motion_notify_event (GtkWidget *widget,
-		     GdkEventMotion *event)
+mouse_change_pos (GtkWidget *widget, gint x, gint y)
 {
 	GtkHTML *html;
 	HTMLEngine *engine;
 	HTMLObject *obj;
-	GdkModifierType mask;
-	const gchar *url;
-	gint x, y;
 	HTMLType type;
 
-	g_return_val_if_fail (widget != NULL, 0);
-	g_return_val_if_fail (GTK_IS_HTML (widget), 0);
-	g_return_val_if_fail (event != NULL, 0);
-
 	html = GTK_HTML (widget);
 	engine = html->engine;
 
-	if (event->is_hint) {
-		gdk_window_get_pointer (GTK_LAYOUT (widget)->bin_window, &x, &y, &mask);
-	} else {
-		x = event->x;
-		y = event->y;
-	}
-
 	obj = html_engine_get_object_at (engine,
 					 x + engine->x_offset, y + engine->y_offset,
 					 NULL, FALSE);
-	if (html->button_pressed) {
+
+	if (html->button_pressed && html->allow_selection) {
 		if (obj) {
 			type = HTML_OBJECT_TYPE (obj);
 
@@ -774,43 +813,40 @@ motion_notify_event (GtkWidget *widget,
 					   html->selection_x1, html->selection_y1,
 					   x + engine->x_offset, y + engine->y_offset,
 					   TRUE);
-		
-		if (html_engine_get_editable (engine))
-			html_engine_jump_at (engine,
-					     event->x + engine->x_offset,
-					     event->y + engine->y_offset);
-		return TRUE;
 	}
 
-	if (obj != NULL)
-		url = html_object_get_url (obj);
-	else
-		url = NULL;
+	if (obj)
+		on_url (widget, obj);
 
-	if (url == NULL) {
-		if (html->pointer_url != NULL) {
-			g_free (html->pointer_url);
-			html->pointer_url = NULL;
-			gtk_signal_emit (GTK_OBJECT (html), signals[ON_URL], NULL);
-		}
+	return TRUE;
+}
 
-		if (obj != NULL && html_object_is_text (obj))
-			gdk_window_set_cursor (widget->window, html->ibeam_cursor);
-		else
-			gdk_window_set_cursor (widget->window, html->arrow_cursor);
+static gint
+motion_notify_event (GtkWidget *widget,
+		     GdkEventMotion *event)
+{
+	HTMLEngine *engine;
+	gint x, y;
+
+	g_return_val_if_fail (widget != NULL, 0);
+	g_return_val_if_fail (GTK_IS_HTML (widget), 0);
+	g_return_val_if_fail (event != NULL, 0);
+
+	if (event->is_hint) {
+		gdk_window_get_pointer (GTK_LAYOUT (widget)->bin_window, &x, &y, NULL);
 	} else {
-		if (html->pointer_url == NULL || strcmp (html->pointer_url, url) != 0) {
-			g_free (html->pointer_url);
-			html->pointer_url = g_strdup (url);
-			gtk_signal_emit (GTK_OBJECT (html), signals[ON_URL], url);
-		}
-
-		if (engine->editable)
-			gdk_window_set_cursor (widget->window, html->ibeam_cursor);
-		else
-			gdk_window_set_cursor (widget->window, html->hand_cursor);
+		x = event->x;
+		y = event->y;
 	}
 
+	if (!mouse_change_pos (widget, x, y))
+		return FALSE;
+
+	engine = GTK_HTML (widget)->engine;
+	if (GTK_HTML (widget)->button_pressed && html_engine_get_editable (engine))
+		html_engine_jump_at (engine,
+				     event->x + engine->x_offset,
+				     event->y + engine->y_offset);
 	return TRUE;
 }
 
@@ -885,7 +921,7 @@ button_press_event (GtkWidget *widget,
 
 	if (!(event->state & GDK_SHIFT_MASK))
 		html_engine_disable_selection (engine);
-	else
+	else if (html->allow_selection)
 		html_engine_select_region (engine,
 					   html->selection_x1, html->selection_y1,
 					   event->x + engine->x_offset, event->y + engine->y_offset,
@@ -925,9 +961,12 @@ static gint
 focus_in_event (GtkWidget *widget,
 		GdkEventFocus *event)
 {
-	GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_FOCUS);
-
-	html_engine_set_focus (GTK_HTML (widget)->engine, TRUE);
+	GtkHTML *html = GTK_HTML (widget);
+	if (!html->iframe_parent) {
+		GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_FOCUS);
+		html_engine_set_focus (html->engine, TRUE);
+	} else
+		gtk_window_set_focus (GTK_WINDOW (html->iframe_parent->window), html->iframe_parent);
 
 	return FALSE;
 }
@@ -936,9 +975,11 @@ static gint
 focus_out_event (GtkWidget *widget,
 		 GdkEventFocus *event)
 {
-	GTK_WIDGET_UNSET_FLAGS (widget, GTK_HAS_FOCUS);
-
-	html_engine_set_focus (GTK_HTML (widget)->engine, FALSE);
+	GtkHTML *html = GTK_HTML (widget);
+	if (!html->iframe_parent) {
+		GTK_WIDGET_UNSET_FLAGS (widget, GTK_HAS_FOCUS);
+		html_engine_set_focus (html->engine, FALSE);
+	}
 
 	return FALSE;
 }
@@ -1103,9 +1144,6 @@ client_notify (GConfClient* client,
 	gchar *tkey;
 
 	g_assert (client == gconf_client);
-
-	printf ("client notify key %s\n", key);
-
 	g_assert (key);
 	tkey = strrchr (key, '/');
 	g_assert (tkey);
@@ -1128,7 +1166,7 @@ init_properties (GtkHTMLClass *klass)
 	klass->properties = gtk_html_class_properties_new ();
 #ifdef GTKHTML_HAVE_GCONF
 	if (gconf_is_initialized ()) {
-		gconf_client = gconf_client_new ();
+		gconf_client = gconf_client_get_default ();
 		if (!gconf_client)
 			g_error ("cannot create gconf_client\n");
 		gconf_client_add_dir (gconf_client, GTK_HTML_GCONF_DIR, GCONF_CLIENT_PRELOAD_ONELEVEL, &gconf_error);
@@ -1328,7 +1366,17 @@ class_init (GtkHTMLClass *klass)
 				GTK_SIGNAL_OFFSET (GtkHTMLClass, command),
 				gtk_marshal_NONE__ENUM,
 				GTK_TYPE_NONE, 1, GTK_TYPE_HTML_COMMAND);
-
+#ifdef GTKHTML_HAVE_PSPELL
+	signals [SPELL_SUGGESTION_REQUEST] =
+		gtk_signal_new ("spell_suggestion_request",
+				GTK_RUN_FIRST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (GtkHTMLClass, spell_suggestion_request),
+				gtk_marshal_NONE__POINTER_POINTER,
+				GTK_TYPE_NONE, 2,
+				GTK_TYPE_POINTER,
+				GTK_TYPE_POINTER);
+#endif
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 
 	object_class->destroy = destroy;
@@ -1448,6 +1496,7 @@ gtk_html_construct (GtkWidget *htmlw)
 	html = GTK_HTML (htmlw);
 
 	html->engine = html_engine_new (htmlw);
+	html->iframe_parent = NULL;
 
 	gtk_signal_connect (GTK_OBJECT (html->engine), "title_changed",
 			    GTK_SIGNAL_FUNC (html_engine_title_changed_cb), html);
@@ -2023,55 +2072,57 @@ move_selection (GtkHTML *html, GtkHTMLCommandType com_type)
 static void
 command (GtkHTML *html, GtkHTMLCommandType com_type)
 {
-	if (!html_engine_get_editable (html->engine))
+	HTMLEngine *e = html->engine;
+
+	if (!html_engine_get_editable (e))
 		return;
 
 	/* printf ("command %d\n", com_type); */
 
 	switch (com_type) {
 	case GTK_HTML_COMMAND_UNDO:
-		html_engine_undo (html->engine);
+		html_engine_undo (e);
 		break;
 	case GTK_HTML_COMMAND_REDO:
-		html_engine_redo (html->engine);
+		html_engine_redo (e);
 		break;
 	case GTK_HTML_COMMAND_COPY:
-		if (html->engine->active_selection)
-			html_engine_copy (html->engine);
+		if (e->active_selection)
+			html_engine_copy (e);
 		break;
 	case GTK_HTML_COMMAND_CUT:
-		html_engine_cut (html->engine, TRUE);
+		html_engine_cut (e, TRUE);
 		break;
 	case GTK_HTML_COMMAND_CUT_LINE:
-		html_engine_cut_line (html->engine, TRUE);
+		html_engine_cut_line (e, TRUE);
 		break;
 	case GTK_HTML_COMMAND_PASTE:
-		if (html->engine->cut_buffer)
-			html_engine_paste (html->engine, TRUE);
+		if (e->cut_buffer)
+			html_engine_paste (e, TRUE);
 		break;
 	case GTK_HTML_COMMAND_INSERT_PARAGRAPH:
-		html_engine_delete_selection (html->engine, TRUE);
-		html_engine_insert (html->engine, "\n", 1);
+		html_engine_delete_selection (e, TRUE);
+		html_engine_insert (e, "\n", 1);
 		break;
 	case GTK_HTML_COMMAND_DELETE:
-		if (html->engine->mark != NULL
-		    && html->engine->mark->position != html->engine->cursor->position)
-			html_engine_delete_selection (html->engine, TRUE);
+		if (e->mark != NULL
+		    && e->mark->position != e->cursor->position)
+			html_engine_delete_selection (e, TRUE);
 		else
-			html_engine_delete (html->engine, 1, TRUE, FALSE);
+			html_engine_delete (e, 1, TRUE, FALSE);
 		break;
 	case GTK_HTML_COMMAND_DELETE_BACK:
-		if (html->engine->mark != NULL
-		    && html->engine->mark->position != html->engine->cursor->position)
-			html_engine_delete_selection (html->engine, TRUE);
+		if (e->mark != NULL
+		    && e->mark->position != e->cursor->position)
+			html_engine_delete_selection (e, TRUE);
 		else
-			html_engine_delete (html->engine, 1, TRUE, TRUE);
+			html_engine_delete (e, 1, TRUE, TRUE);
 		break;
 	case GTK_HTML_COMMAND_SET_MARK:
-		html_engine_set_mark (html->engine);
+		html_engine_set_mark (e);
 		break;
 	case GTK_HTML_COMMAND_DISABLE_SELECTION:
-		html_engine_disable_selection (html->engine);
+		html_engine_disable_selection (e);
 		break;
 	case GTK_HTML_COMMAND_TOGGLE_BOLD:
 		gtk_html_toggle_font_style (html, GTK_HTML_FONT_STYLE_BOLD);
@@ -2107,10 +2158,10 @@ command (GtkHTML *html, GtkHTMLCommandType com_type)
 		gtk_html_set_font_style (html, ~GTK_HTML_FONT_STYLE_SIZE_MASK, GTK_HTML_FONT_STYLE_SIZE_7);
 		break;
 	case GTK_HTML_COMMAND_SIZE_INCREASE:
-		html_engine_font_size_inc_dec (html->engine, TRUE);
+		html_engine_font_size_inc_dec (e, TRUE);
 		break;
 	case GTK_HTML_COMMAND_SIZE_DECREASE:
-		html_engine_font_size_inc_dec (html->engine, FALSE);
+		html_engine_font_size_inc_dec (e, FALSE);
 		break;
 	case GTK_HTML_COMMAND_ALIGN_LEFT:
 		gtk_html_set_paragraph_alignment (html, GTK_HTML_PARAGRAPH_ALIGNMENT_LEFT);
@@ -2176,14 +2227,35 @@ command (GtkHTML *html, GtkHTMLCommandType com_type)
 		move_selection (html, com_type);
 		break;
 	case GTK_HTML_COMMAND_CAPITALIZE_WORD:
-		html_engine_capitalize_word (html->engine);
+		html_engine_capitalize_word (e);
 		break;
 	case GTK_HTML_COMMAND_UPCASE_WORD:
-		html_engine_upcase_downcase_word (html->engine, TRUE);
+		html_engine_upcase_downcase_word (e, TRUE);
 		break;
 	case GTK_HTML_COMMAND_DOWNCASE_WORD:
-		html_engine_upcase_downcase_word (html->engine, FALSE);
+		html_engine_upcase_downcase_word (e, FALSE);
 		break;
+#ifdef GTKHTML_HAVE_PSPELL
+	case GTK_HTML_COMMAND_SPELL_SUGGEST:
+		if (!html_engine_word_is_valid (e))
+			gtk_signal_emit (GTK_OBJECT (html), signals [SPELL_SUGGESTION_REQUEST],
+					 e->spell_checker, html_engine_get_word (e));
+		break;
+	case GTK_HTML_COMMAND_SPELL_PERSONAL_DICTIONARY_ADD:
+	case GTK_HTML_COMMAND_SPELL_SESSION_DICTIONARY_ADD: {
+		gchar *word = html_engine_get_word (e);
+		if (word) {
+			if (com_type == GTK_HTML_COMMAND_SPELL_PERSONAL_DICTIONARY_ADD)
+				pspell_manager_add_to_personal (e->spell_checker, word);
+			else
+				pspell_manager_add_to_session (e->spell_checker, word);
+			pspell_manager_save_all_word_lists (e->spell_checker);
+			html_engine_spell_check (e);
+			gtk_widget_queue_draw (GTK_WIDGET (html));
+		}
+		break;
+	}
+#endif
 	default:
 		return;
 	}
@@ -2279,8 +2351,9 @@ load_keybindings (GtkHTMLClass *klass)
 					  gtk_type_name (GTK_OBJECT_CLASS (klass)->type),
 					  GTK_PATH_PRIO_GTK);
 		g_free (name);
-	} else
-		g_warning (_("Couldn't find keybinding file -- %s"), rcfile);
+	}
+	/* else
+		g_warning (_("Couldn't find keybinding file -- %s"), rcfile); */
 	g_free (rcfile);
 
 	binding_set = gtk_binding_set_by_class (klass);
@@ -2356,3 +2429,11 @@ load_keybindings (GtkHTMLClass *klass)
 	BCOM (0, Delete, DELETE);
 	BCOM (0, KP_Delete, DELETE);
 }
+
+void
+gtk_html_set_iframe_parent (GtkHTML *html, GtkWidget *parent)
+{
+	g_assert (GTK_IS_HTML (parent));
+
+	html->iframe_parent = parent;
+}
