@@ -26,6 +26,7 @@
 /* WARNING: it must always be the child of a clue.  */
 
 #include <config.h>
+#include <ctype.h>
 #include <string.h>
 #include "htmlclue.h"
 #include "htmlclueflow.h"
@@ -246,107 +247,45 @@ set_max_width (HTMLObject *o,
 	}
 }
 
-
-static gboolean
-should_break (HTMLObject *obj)
-{
-	/* end: last char of obj->prev, beg: beggining char of obj */
-	guchar end=' ', beg=' ';
-	HTMLObject *next;
-
-	if (!obj)
-		return TRUE;
-	next = obj->next;
-
-	/* it is on the beginning of Clue */
-	if (!next)
-		return TRUE;
-
-	/* skip prev Master */
-	if ((HTML_OBJECT_TYPE (obj) == HTML_TYPE_TEXTMASTER ||
-	     HTML_OBJECT_TYPE (obj) == HTML_TYPE_LINKTEXTMASTER) &&
-	    HTML_OBJECT_TYPE (next)  == HTML_TYPE_TEXTSLAVE  &&
-	    HTML_TEXT_SLAVE (next)->owner == HTML_TEXT_MASTER (obj)) {
-		next = next->next;
-		/* it is Master->Slave */
-		if (!next)
-			return TRUE;
-	}
-
-	/* get end */
-	if (HTML_OBJECT_TYPE (obj) == HTML_TYPE_TEXTSLAVE) {
-
-		HTMLTextSlave *slave = HTML_TEXT_SLAVE (obj);
-		end = HTML_TEXT (slave->owner)->text [slave->posStart + slave->posLen - 1];
-	} else
-		if (HTML_OBJECT_TYPE (obj) == HTML_TYPE_TEXTMASTER ||
-		    HTML_OBJECT_TYPE (obj) == HTML_TYPE_LINKTEXTMASTER) {
-
-			HTMLText *master = HTML_TEXT (obj);
-			if (master->text_len > 0)
-				end = master->text [master->text_len - 1];
-			else
-				/* XXX is this the correct return value? */
-				end = ' ';
-		}
-
-	/* get beg */
-	if (HTML_OBJECT_TYPE (next) == HTML_TYPE_TEXTSLAVE) {
-		HTMLTextSlave *slave = HTML_TEXT_SLAVE (next);
-		beg = HTML_TEXT (slave->owner)->text [slave->posStart];
-	} else if (HTML_OBJECT_TYPE (next) == HTML_TYPE_TEXTMASTER ||
-		   HTML_OBJECT_TYPE (next) == HTML_TYPE_LINKTEXTMASTER)
-		beg = HTML_TEXT (next)->text [0];
-
-	/* printf ("end: %c beg: %c ret: %d\n", end, beg, end == ' ' || beg == ' '); */
-
-	return end == ' ' || beg == ' ';
-}
-
 static gint
 calc_min_width (HTMLObject *o,
 		HTMLPainter *painter)
 {
-	HTMLObject *obj, *co;
-	gint w, cw;
-	gint cl = 0;
-	gint tempMinWidth;
-	gboolean is_pre, is_nowrap;
+	HTMLObject *cur;
+	gint min_width = 0;
+	gint w = 0;
+	gboolean add;
 
-	tempMinWidth = 0;
-	w = 0;
-	is_pre = (HTML_CLUEFLOW (o)->style == HTML_CLUEFLOW_STYLE_PRE);
-	is_nowrap = (HTML_CLUEFLOW (o)->style == HTML_CLUEFLOW_STYLE_NOWRAP);
+	add = (HTML_CLUEFLOW (o)->style == HTML_CLUEFLOW_STYLE_PRE
+	       || HTML_CLUEFLOW (o)->style == HTML_CLUEFLOW_STYLE_NOWRAP);
 
-	for (obj = HTML_CLUE (o)->head; obj != 0; obj = obj->next) {
-		obj->nb_width = html_object_calc_min_width (obj, painter);
-		w += obj->nb_width;
-
-		if ((is_pre || is_nowrap)
-		    && ! (obj->flags & HTML_OBJECT_FLAG_NEWLINE)
-		    && obj->next != NULL)
-			continue;
-
-		if (w > tempMinWidth)
-			tempMinWidth = w;
-
-		if (should_break (obj) || !obj->next) {
-			co = obj->prev;
-			cw = 0;
-			while (cl) {
-				cw += co->nb_width;
-				co->nb_width = cw;
-				co = co->prev;
-				cl--;
-			}
-			w  = 0;
-		} else
-			cl++;
+	cur = HTML_CLUE (o)->head;
+	while (cur) {
+		w += (add) ? html_object_calc_preferred_width (cur, painter) : html_object_calc_min_width (cur, painter);
+		if (!add || cur->flags & HTML_OBJECT_FLAG_NEWLINE || !cur->next) {
+			if (min_width < w) min_width = w;
+			w = 0;
+		}
+		cur = cur->next;
 	}
 
-	tempMinWidth += get_indent (HTML_CLUEFLOW (o), painter);
+	/* printf ("flow min width: %d\n", min_width + get_indent (HTML_CLUEFLOW (o), painter)); */
 
-	return tempMinWidth;
+	return min_width + get_indent (HTML_CLUEFLOW (o), painter);
+}
+
+static gint
+set_line_x (HTMLObject **obj, HTMLObject *run, gint x, gboolean *changed)
+{
+	while (*obj != run) {
+		if ((*obj)->x != x) {
+			(*obj)->x = x;   
+			*changed = TRUE;
+		}
+		x   += (*obj)->width;
+		*obj = (*obj)->next;
+	}
+	return x;
 }
 
 /* EP CHECK: should be mostly OK.  */
@@ -362,7 +301,6 @@ calc_size (HTMLObject *o,
 	HTMLClearType clear;
 	gboolean newLine;
 	gboolean firstLine;
-	gboolean is_pre;
 	gint lmargin, rmargin;
 	gint indent;
 	gint oldy;
@@ -387,11 +325,6 @@ calc_size (HTMLObject *o,
 	clear = HTML_CLEAR_NONE;
 	indent = get_indent (HTML_CLUEFLOW (o), painter);
 
-	if (HTML_CLUEFLOW (o)->style == HTML_CLUEFLOW_STYLE_PRE)
-		is_pre = TRUE;
-	else
-		is_pre = FALSE;
-
 	o->ascent = 0;
 	o->descent = 0;
 	o->width = 0;
@@ -500,21 +433,18 @@ calc_size (HTMLObject *o,
 				HTMLVAlignType valign;
 				gint width_left;
 
-				if (is_pre)
-					width_left = -1;
-				else
-					width_left = rmargin - runWidth - w;
-
-				if (! is_pre && run != obj && runWidth + run->nb_width > rmargin - lmargin)
-					break;
-
+				width_left = rmargin - runWidth - w;
 				fit = html_object_fit_line (run,
 							    painter,
 							    w + runWidth == lmargin,
-							    obj == line,
+							    run == line || (HTML_OBJECT_TYPE (run) == HTML_TYPE_TEXTSLAVE
+									    && HTML_OBJECT (HTML_TEXT_SLAVE (run)->owner)
+									    == line
+									    && !HTML_TEXT_SLAVE (run)->posStart),
 							    width_left);
 
 				if (fit == HTML_FIT_NONE) {
+					w = set_line_x (&obj, run, w, &changed);
 					newLine = TRUE;
 					break;
 				}
@@ -565,12 +495,8 @@ calc_size (HTMLObject *o,
 					/* Implicit separator */
 					break;
 				}
-
-				if (! is_pre && (runWidth > rmargin - lmargin))
-					break;
 			}
 
-
 			if (! newLine) {
 				gint new_y, new_lmargin, new_rmargin;
 
@@ -615,14 +541,7 @@ calc_size (HTMLObject *o,
 					newLine = FALSE;
 					clear = HTML_CLEAR_NONE;
 				} else {
-					while (obj != run) {
-						if (obj->x != w) {
-							obj->x = w;   
-							changed = TRUE;
-						}
-						w += obj->width;
-						obj = obj->next;
-					}
+					w = set_line_x (&obj, run, w, &changed);
 					/* we've used up this line so insert a newline */
 					newLine = TRUE;
 				}
@@ -638,14 +557,8 @@ calc_size (HTMLObject *o,
 		/* if we need a new line, or all objects have been processed
 		   and need to be aligned. */
 		if ( newLine || !obj) {
-			HTMLObject *eol;
 			int extra;
 
-			/* remove trailing spaces */
-			eol = (obj) ? obj : clue->tail;
-			if (HTML_OBJECT_TYPE (eol) == HTML_TYPE_TEXTSLAVE)
-				w -= html_text_slave_trail_space_width (HTML_TEXT_SLAVE (eol), painter);
-
 			extra = 0;
 
 			if (w > o->width)
@@ -954,10 +867,10 @@ get_tag (HTMLClueFlow *flow)
 {
 	switch (flow->style) {
 	case HTML_CLUEFLOW_STYLE_ITEMDOTTED:
-		return "UL";
+		return "ul";
 	case HTML_CLUEFLOW_STYLE_ITEMROMAN:
 	case HTML_CLUEFLOW_STYLE_ITEMDIGIT:
-		return "OL";
+		return "ol";
 	case HTML_CLUEFLOW_STYLE_NORMAL:
 	case HTML_CLUEFLOW_STYLE_H1:
 	case HTML_CLUEFLOW_STYLE_H2:
@@ -969,7 +882,7 @@ get_tag (HTMLClueFlow *flow)
 	case HTML_CLUEFLOW_STYLE_PRE:
 	case HTML_CLUEFLOW_STYLE_NOWRAP:
 	default:
-		return "BLOCKQUOTE";
+		return "blockquote";
 	}
 }
 
@@ -1024,7 +937,7 @@ write_pre_tags (HTMLClueFlow *self,
 	prev = HTML_CLUEFLOW (HTML_OBJECT (self)->prev);
 	if (prev != NULL && prev->level == self->level && prev->style == self->style) {
 		if (! is_item (self))
-			return html_engine_save_output_string (state, "<BR>\n");
+			return html_engine_save_output_string (state, "<br>\n");
 		else
 			return TRUE;
 	}
@@ -1069,25 +982,25 @@ get_tag_for_style (const HTMLClueFlow *flow)
 	case HTML_CLUEFLOW_STYLE_NORMAL:
 		return NULL;
 	case HTML_CLUEFLOW_STYLE_H1:
-		return "H1";
+		return "h1";
 	case HTML_CLUEFLOW_STYLE_H2:
-		return "H2";
+		return "h2";
 	case HTML_CLUEFLOW_STYLE_H3:
-		return "H3";
+		return "h3";
 	case HTML_CLUEFLOW_STYLE_H4:
-		return "H4";
+		return "h4";
 	case HTML_CLUEFLOW_STYLE_H5:
-		return "H5";
+		return "h5";
 	case HTML_CLUEFLOW_STYLE_H6:
-		return "H6";
+		return "h6";
 	case HTML_CLUEFLOW_STYLE_ADDRESS:
-		return "ADDRESS";
+		return "address";
 	case HTML_CLUEFLOW_STYLE_PRE:	
-		return "PRE";
+		return "pre";
 	case HTML_CLUEFLOW_STYLE_ITEMDOTTED:
 	case HTML_CLUEFLOW_STYLE_ITEMROMAN:
 	case HTML_CLUEFLOW_STYLE_ITEMDIGIT:
-		return "LI";
+		return "li";
 	default:
 		g_warning ("Unknown HTMLClueFlowStyle %d", flow->style);
 		return NULL;
@@ -1099,13 +1012,13 @@ halign_to_string (HTMLHAlignType halign)
 {
 	switch (halign) {
 	case HTML_HALIGN_RIGHT:
-		return "RIGHT";
+		return "right";
 	case HTML_HALIGN_CENTER:
-		return "CENTER";
+		return "center";
 	case HTML_HALIGN_LEFT:
 	case HTML_HALIGN_NONE:
 	default:
-		return "LEFT";
+		return "left";
 	}
 }
 
@@ -1155,7 +1068,7 @@ save (HTMLObject *self,
 	
 	/* Alignment tag.  */
 	if (halign != HTML_HALIGN_NONE && halign != HTML_HALIGN_LEFT) {
-		if (! html_engine_save_output_string (state, "<DIV ALIGN=%s>\n", halign_to_string (halign)))
+		if (! html_engine_save_output_string (state, "<div alignx=%s>\n", halign_to_string (halign)))
 			return FALSE;
 	}
 
@@ -1176,7 +1089,7 @@ save (HTMLObject *self,
 
 	/* Close alignment tag.  */
 	if (halign != HTML_HALIGN_NONE && halign != HTML_HALIGN_LEFT) {
-		if (! html_engine_save_output_string (state, "</DIV>\n"))
+		if (! html_engine_save_output_string (state, "</div>\n"))
 			return FALSE;
 	} else if (tag != NULL
 		   && HTML_CLUEFLOW (self)->style != HTML_CLUEFLOW_STYLE_PRE) {
@@ -1494,9 +1407,8 @@ search (HTMLObject *obj, HTMLSearch *info)
 			}
 		} else {
 			html_search_push (info, cur);
-			if (html_object_search (cur, info)) {
+			if (html_object_search (cur, info))
 				return TRUE;
-			}
 			html_search_pop (info);
 			cur = (info->forward) ? cur->next : cur->prev;
 		}
@@ -1843,3 +1755,153 @@ html_clueflow_remove_text_slaves (HTMLClueFlow *flow)
 		}
 	}
 }
+
+#ifdef GTKHTML_HAVE_PSPELL
+
+/* spell checking */
+
+static gchar *
+get_text (HTMLClue *clue)
+{
+	HTMLObject *obj;
+	guint len = 0;
+	gchar *text, *ct;
+
+	obj = clue->head;
+	while (obj) {
+		if (html_object_is_text (obj))
+			len += HTML_TEXT (obj)->text_len;
+		else if (HTML_OBJECT_TYPE (obj) != HTML_TYPE_TEXTSLAVE)
+			len++;
+		obj = obj->next;
+	}
+
+	ct  = text = g_malloc (len+1);
+	text [len] = 0;
+	obj = clue->head;
+	while (obj) {
+		if (html_object_is_text (obj)) {
+			strcpy (ct, HTML_TEXT (obj)->text);
+			ct += HTML_TEXT (obj)->text_len;
+		} else if (HTML_OBJECT_TYPE (obj) != HTML_TYPE_TEXTSLAVE) {
+			*ct = ' ';
+			ct++;
+		}
+		obj = obj->next;
+	}
+
+	/* printf ("get_text: \"%s\"\n", text); */
+
+	return text;
+}
+
+static HTMLObject *
+next_obj (HTMLObject *obj, guint *off, gboolean *is_text)
+{
+	*off += (*is_text)
+		? HTML_TEXT (obj)->text_len
+		: ((HTML_OBJECT_TYPE (obj) != HTML_TYPE_TEXTSLAVE) ? 1 : 0);
+	obj = obj->next;
+	if (obj && (*is_text = html_object_is_text (obj)))
+		html_text_spell_errors_clear (HTML_TEXT (obj));
+
+	return obj;
+}
+
+static HTMLObject *
+spell_check_word (HTMLObject *obj, const gchar *text, const gchar *word, guint *off)
+{
+	guint w_off;
+	guint len = strlen (word);
+	gboolean is_text;
+
+	/* printf ("[not in dictionary off: %d]\n", word - text); */
+	is_text = html_object_is_text (obj);
+	w_off   = word - text;
+	while (obj && (!is_text || (is_text && *off + HTML_TEXT (obj)->text_len <= w_off)))
+		obj = next_obj (obj, off, &is_text);
+
+	/* printf ("is_text: %d len: %d obj: %p\n", is_text, len, obj); */
+	if (obj && is_text) {
+		guint tlen;
+		guint toff;
+
+		while (len) {
+			toff  = w_off - *off;
+			tlen  = MIN (HTML_TEXT (obj)->text_len - toff, len);
+			g_assert (!strncmp (text + w_off, HTML_TEXT (obj)->text + toff, tlen));
+			/* printf ("add spell error - word: %s beg: %s len: %d\n",
+			   word, HTML_TEXT (obj)->text + toff, tlen); */
+			html_text_spell_errors_add (HTML_TEXT (obj),
+						    toff, tlen);
+			len   -= tlen;
+			w_off += tlen;
+			if (len)
+				do obj = next_obj (obj, off, &is_text); while (obj && !is_text);
+			g_assert (!len || obj);
+		}
+	}
+
+	return obj;
+}
+
+void
+html_clueflow_spell_check (HTMLClueFlow *flow, HTMLEngine *e)
+{
+	HTMLObject *obj;
+	HTMLClue *clue;
+	guint off;
+	gchar *text, *ct, *word;
+
+	if (!e->spell_checker)
+		return;
+
+	off  = 0;
+	clue = HTML_CLUE (flow);
+	text = get_text (clue);
+	obj  = clue->head;
+	if (obj && html_object_is_text (obj))
+		html_text_spell_errors_clear (HTML_TEXT (obj));
+
+	if (text) {
+		ct = text;
+		while (*ct) {
+			/* find begin of word */
+			while (*ct && !isalpha (*ct)) ct++;
+			word = ct;
+			/* find end of word */
+			while (isalpha (*ct)) ct++;
+
+			/* test if we have found word */
+			if (word != ct) {
+				gint result;
+				gchar bak;
+
+				bak = *ct;
+				*ct = 0;
+				/* printf ("going to test word: \"%s\"\n", word); */
+				result = pspell_manager_check (e->spell_checker, word);
+
+				if (result == 1) {
+					gboolean is_text = (obj) ? html_object_is_text (obj) : FALSE;
+					while (obj && (!is_text || (is_text && off + HTML_TEXT (obj)->text_len < ct - text)))
+						obj = next_obj (obj, &off, &is_text);
+				} else {
+					if (result == 0) {
+						if (obj)
+							obj = spell_check_word (obj, text, word, &off);
+					} else if (result == -1)
+						g_warning ("pspell error: %s\n",
+							   pspell_manager_error_message (e->spell_checker));
+					else
+						g_assert_not_reached ();
+				}
+
+				*ct = bak;
+				if (ct) ct++;
+			}
+		}
+	}
+}
+
+#endif /* GTKHTML_HAVE_PSPELL */
