@@ -132,6 +132,7 @@ do_layout (HTMLObject *o,
 		   object.  */
 		remove_aligned_by_parent (cluev, clue->curr);
 	} else {
+		o->width = 0;
 		o->ascent = cluev->padding;
 		o->descent = 0;
 		clue->curr = clue->head;
@@ -145,7 +146,7 @@ do_layout (HTMLObject *o,
 		if (calc_size)
 			changed |= html_object_calc_size (clue->curr, painter);
 
-		if (clue->curr->width > o->width - 2 * cluev->padding)
+		if (o->width < clue->curr->width + 2 * cluev->padding)
 			o->width = clue->curr->width + 2 * cluev->padding;
 
 		o->ascent += clue->curr->ascent + clue->curr->descent;
@@ -226,18 +227,10 @@ static void
 set_max_width (HTMLObject *o, HTMLPainter *painter, gint max_width)
 {
 	HTMLObject *obj;
-	
-	if (!(o->flags & HTML_OBJECT_FLAG_FIXEDWIDTH)) {
-		o->max_width = max_width;
-		if (o->percent > 0)
-			o->width = max_width * o->percent / 100;
-		else 
-			o->width = o->max_width;
-	}
-
-	for (obj = HTML_CLUE (o)->head; obj != 0; obj = obj->next)
-		html_object_set_max_width (obj, painter, o->width);
 
+	o->max_width = max_width;
+	for (obj = HTML_CLUE (o)->head; obj != NULL; obj = obj->next)
+		html_object_set_max_width (obj, painter, o->max_width);
 }
 
 static void
@@ -249,8 +242,8 @@ reset (HTMLObject *clue)
 
 	HTML_OBJECT_CLASS (&html_clue_class)->reset (clue);
 
-	cluev->align_left_list = 0;
-	cluev->align_right_list = 0;
+	cluev->align_left_list = NULL;
+	cluev->align_right_list = NULL;
 }
 
 static void
@@ -489,13 +482,12 @@ get_right_margin (HTMLObject *self, gint y)
 	gint margin;
 	
 	cluev = HTML_CLUEV (self);
-	margin = MAX (self->max_width, self->width);
+	margin = self->max_width;
 
 	for (aclue = cluev->align_right_list;
 	     aclue != NULL;
 	     aclue = cluev_next_aligned (aclue)) {
-		if ((aclue->y - aclue->ascent + aclue->parent->y
-		     - aclue->parent->ascent) <= y
+		if ((aclue->y - aclue->ascent + aclue->parent->y - aclue->parent->ascent) <= y
 		    && aclue->y + aclue->parent->y - aclue->parent->ascent > y)
 			margin = aclue->x;
 	}
@@ -781,15 +773,11 @@ html_cluev_class_init (HTMLClueVClass *klass,
 	HTMLObjectClass *object_class;
 	HTMLClueClass *clue_class;
 
-	/* FIXME destroy!? */
-
 	object_class = HTML_OBJECT_CLASS (klass);
 	clue_class = HTML_CLUE_CLASS (klass);
 
 	html_clue_class_init (clue_class, type, size);
 
-	/* FIXME destroy */
-
 	object_class->copy = copy;
 	object_class->calc_size = calc_size;
 	object_class->relayout = relayout;
@@ -815,7 +803,7 @@ void
 html_cluev_init (HTMLClueV *cluev,
 		 HTMLClueVClass *klass,
 		 gint x, gint y,
-		 gint max_width, gint percent)
+		 gint percent)
 {
 	HTMLObject *object;
 	HTMLClue *clue;
@@ -827,20 +815,8 @@ html_cluev_init (HTMLClueV *cluev,
 
 	object->x = x;
 	object->y = y;
-	object->max_width = max_width;
 	object->percent = percent;
 
-	if (object->percent > 0) {
-		object->width = max_width * percent / 100;
-		object->flags &= ~HTML_OBJECT_FLAG_FIXEDWIDTH;
-	}
-	else if (percent < 0) {
-		object->width = max_width;
-		object->flags &= ~HTML_OBJECT_FLAG_FIXEDWIDTH;
-	}
-	else
-		object->width = max_width;
-
 	clue->valign = HTML_VALIGN_BOTTOM;
 	clue->halign = HTML_HALIGN_LEFT;
 	clue->head = clue->tail = clue->curr = 0;
@@ -851,12 +827,12 @@ html_cluev_init (HTMLClueV *cluev,
 }
 
 HTMLObject *
-html_cluev_new (gint x, gint y, gint max_width, gint percent)
+html_cluev_new (gint x, gint y, gint percent)
 {
 	HTMLClueV *cluev;
 
 	cluev = g_new (HTMLClueV, 1);
-	html_cluev_init (cluev, &html_cluev_class, x, y, max_width, percent);
+	html_cluev_init (cluev, &html_cluev_class, x, y, percent);
 	
 	return HTML_OBJECT (cluev);
 }
