@@ -22,7 +22,9 @@
 #include "gtkhtmldebug.h"
 #include "htmlclue.h"
 #include "htmltextmaster.h"
+#include "htmlengine-edit.h"
 #include "htmlengine-edit-movement.h"
+#include "htmlengine-edit-selection-updater.h"
 
 #include "htmlengine-edit-fontstyle.h"
 
@@ -99,16 +101,21 @@ update_cursor_for_merge (HTMLText *text,
    and makes sure that @cursor is still valid (i.e. still points to a
    valid object) after this.  */
 static void
-merge_safely (HTMLText *text,
+merge_safely (HTMLEngine *engine,
+	      HTMLText *text,
 	      HTMLText *other,
-	      gboolean prepend,
-	      HTMLCursor *cursor,
-	      HTMLCursor *mark)
+	      gboolean prepend)
 {
+	HTMLCursor *cursor, *mark;
+
+	cursor = engine->cursor;
+	mark = engine->mark;
+
 	html_text_master_destroy_slaves (HTML_TEXT_MASTER (text));
 	html_text_master_destroy_slaves (HTML_TEXT_MASTER (other));
 
 	update_cursor_for_merge (text, other, prepend, cursor);
+
 	if (mark != NULL)
 		update_cursor_for_merge (text, other, prepend, mark);
 
@@ -151,9 +158,8 @@ split_safely (HTMLText *text,
 /* This handles merging of consecutive text elements with the same
    properties.  */
 static guint
-merge_backward (HTMLObject *object,
-		HTMLCursor *cursor,
-		HTMLCursor *mark)
+merge_backward (HTMLEngine *engine,
+		HTMLObject *object)
 {
 	GtkHTMLFontStyle font_style;
 	HTMLObject *p, *pprev;
@@ -171,7 +177,7 @@ merge_backward (HTMLObject *object,
 			   && HTML_OBJECT_TYPE (object) == HTML_OBJECT_TYPE (p)
 			   && html_text_check_merge (HTML_TEXT (object), HTML_TEXT (p))) {
 			total_merge += HTML_TEXT (object)->text_len;
-			merge_safely (HTML_TEXT (object), HTML_TEXT (p), TRUE, cursor, mark);
+			merge_safely (engine, HTML_TEXT (object), HTML_TEXT (p), TRUE);
 		} else {
 			break;
 		}
@@ -181,14 +187,17 @@ merge_backward (HTMLObject *object,
 }
 
 static guint
-merge_forward (HTMLObject *object,
-	       HTMLCursor *cursor,
-	       HTMLCursor *mark)
+merge_forward (HTMLEngine *engine,
+	       HTMLObject *object)
 {
 	GtkHTMLFontStyle font_style;
+	HTMLCursor *cursor, *mark;
 	HTMLObject *p, *pnext;
 	guint total_merge;
 
+	cursor = engine->cursor;
+	mark = engine->mark;
+
 	font_style = HTML_TEXT (object)->font_style;
 	total_merge = 0;
 
@@ -201,7 +210,7 @@ merge_forward (HTMLObject *object,
 			   && HTML_OBJECT_TYPE (object) == HTML_OBJECT_TYPE (p)
 			   && html_text_check_merge (HTML_TEXT (object), HTML_TEXT (p))) {
 			total_merge += HTML_TEXT (object)->text_len;
-			merge_safely (HTML_TEXT (object), HTML_TEXT (p), FALSE, cursor, mark);
+			merge_safely (engine, HTML_TEXT (object), HTML_TEXT (p), FALSE);
 		} else {
 			break;
 		}
@@ -334,7 +343,7 @@ set_font_style (HTMLEngine *engine,
 		   same font style.  We don't want to have contiguous equal text
 		   elements, so we merge more stuff here.  */
 
-		total_merged = merge_backward (p, engine->cursor, engine->mark);
+		total_merged = merge_backward (engine, p);
 		if (backwards) {
 			if (total_merged > count)
 				count = 0;
@@ -342,7 +351,7 @@ set_font_style (HTMLEngine *engine,
 				count -= total_merged;
 		}
 
-		total_merged = merge_forward (p, engine->cursor, engine->mark);
+		total_merged = merge_forward (engine, p);
 		if (! backwards) {
 			if (total_merged > count)
 				count = 0;
@@ -378,19 +387,21 @@ set_font_style (HTMLEngine *engine,
 
 	if (backwards) {
 		if (engine->cursor->object->prev != NULL)
-			merge_forward (engine->cursor->object->prev, engine->cursor, engine->mark);
+			merge_forward (engine, engine->cursor->object->prev);
 		if (p != NULL)
-			merge_backward (p, engine->cursor, engine->mark);
+			merge_backward (engine, p);
 	} else {
-		merge_backward (engine->cursor->object, engine->cursor, engine->mark);
+		merge_backward (engine, engine->cursor->object);
 		if (p != NULL)
-			merge_forward (p, engine->cursor, engine->mark);
+			merge_forward (engine, p);
 	}
 
 	html_object_relayout (p->parent->parent, engine, p->parent);
 	html_engine_queue_draw (engine, p->parent);
 
 	html_cursor_normalize (engine->cursor);
+	html_engine_edit_selection_updater_cursor_changed (engine->selection_updater,
+							   engine->cursor);
 
 	if (! want_undo)
 		return NULL;
@@ -414,6 +425,8 @@ set_font_style_in_selection (HTMLEngine *engine,
 	g_return_if_fail (engine->clue != NULL);
 	g_return_if_fail (engine->mark != NULL);
 
+	html_engine_edit_selection_updater_update_now (engine->selection_updater);
+
 #ifdef PARANOID_DEBUG
 	g_print ("Tree before changing font style:\n");
 	gtk_html_debug_dump_tree_simple (engine->clue, 2);
@@ -670,10 +683,10 @@ get_font_style_from_selection (HTMLEngine *engine)
 	g_return_val_if_fail (engine->clue != NULL, GTK_HTML_FONT_STYLE_DEFAULT);
 	g_assert (engine->mark != NULL);
 
-	printf ("%s mark %p,%d cursor %p,%d\n",
+	/* printf ("%s mark %p,%d cursor %p,%d\n",
 		__FUNCTION__,
 		engine->mark, engine->mark->position,
-		engine->cursor, engine->cursor->position);
+		engine->cursor, engine->cursor->position); */
 
 	if (engine->mark->position < engine->cursor->position)
 		backwards = TRUE;
@@ -710,6 +723,95 @@ get_font_style_from_selection (HTMLEngine *engine)
 	return style & ~conflicts;
 }
 
+static HTMLColor *
+get_color_from_selection (HTMLEngine *engine)
+{
+	HTMLColor *color = NULL;
+	gboolean backwards;
+	HTMLObject *p;
+
+	g_return_val_if_fail (engine->clue != NULL, NULL);
+	g_assert (engine->mark != NULL);
+
+	backwards = (engine->mark->position < engine->cursor->position) ? TRUE : FALSE;
+
+	p = engine->cursor->object;
+	while (1) {
+		if (html_object_is_text (p) && p->selected) {
+			color = HTML_TEXT (p)->color;
+			break;
+		}
+
+		if (p == engine->mark->object)
+			break;
+
+		if (backwards)
+			p = html_object_prev_for_cursor (p);
+		else
+			p = html_object_next_for_cursor (p);
+
+		g_assert (p != NULL);
+	}
+
+	return color;
+}
+
+GtkHTMLFontStyle
+html_engine_get_document_font_style (HTMLEngine *engine)
+{
+	HTMLObject *curr;
+
+	g_return_val_if_fail (engine != NULL, GTK_HTML_FONT_STYLE_DEFAULT);
+	g_return_val_if_fail (HTML_IS_ENGINE (engine), GTK_HTML_FONT_STYLE_DEFAULT);
+	g_return_val_if_fail (engine->editable, GTK_HTML_FONT_STYLE_DEFAULT);
+
+	curr = engine->cursor->object;
+
+	if (curr == NULL)
+		return GTK_HTML_FONT_STYLE_DEFAULT;
+	else if (! html_object_is_text (curr))
+		return GTK_HTML_FONT_STYLE_DEFAULT;
+	else if (! engine->active_selection)
+		return HTML_TEXT (curr)->font_style;
+	else
+		return get_font_style_from_selection (engine);
+}
+
+HTMLColor *
+html_engine_get_document_color (HTMLEngine *engine)
+{
+	HTMLObject *curr;
+
+	g_return_val_if_fail (engine != NULL, NULL);
+	g_return_val_if_fail (HTML_IS_ENGINE (engine), NULL);
+	g_return_val_if_fail (engine->editable, NULL);
+
+	curr = engine->cursor->object;
+
+	if (curr == NULL)
+		return NULL;
+	else if (! html_object_is_text (curr))
+		return NULL;
+	else if (! engine->active_selection)
+		return HTML_TEXT (curr)->color;
+	else
+		return get_color_from_selection (engine);
+}
+
+GtkHTMLFontStyle
+html_engine_get_font_style (HTMLEngine *engine)
+{
+	return (engine->insertion_font_style == GTK_HTML_FONT_STYLE_DEFAULT)
+		? html_engine_get_document_font_style (engine)
+		: engine->insertion_font_style;
+}
+
+HTMLColor *
+html_engine_get_color (HTMLEngine *engine)
+{
+	return engine->insertion_color;
+}
+
 /**
  * html_engine_update_insertion_font_style:
  * @engine: An HTMLEngine
@@ -722,23 +824,9 @@ get_font_style_from_selection (HTMLEngine *engine)
 gboolean
 html_engine_update_insertion_font_style (HTMLEngine *engine)
 {
-	HTMLObject *curr;
 	GtkHTMLFontStyle new_style;
 
-	g_return_val_if_fail (engine != NULL, GTK_HTML_FONT_STYLE_DEFAULT);
-	g_return_val_if_fail (HTML_IS_ENGINE (engine), GTK_HTML_FONT_STYLE_DEFAULT);
-	g_return_val_if_fail (engine->editable, GTK_HTML_FONT_STYLE_DEFAULT);
-
-	curr = engine->cursor->object;
-
-	if (curr == NULL)
-		new_style = GTK_HTML_FONT_STYLE_DEFAULT;
-	else if (! html_object_is_text (curr))
-		new_style = GTK_HTML_FONT_STYLE_DEFAULT;
-	else if (! engine->active_selection)
-		new_style = HTML_TEXT (curr)->font_style;
-	else
-		new_style = get_font_style_from_selection (engine);
+	new_style = html_engine_get_document_font_style (engine);
 
 	if (new_style != engine->insertion_font_style) {
 		engine->insertion_font_style = new_style;
@@ -748,6 +836,32 @@ html_engine_update_insertion_font_style (HTMLEngine *engine)
 	return FALSE;
 }
 
+/**
+ * html_engine_update_insertion_style:
+ * @engine: An HTMLEngine
+ * 
+ * Update @engine's current insertion font style/color according to the
+ * current selection and cursor position.
+ * 
+ * Return value: 
+ **/
+gboolean
+html_engine_update_insertion_color (HTMLEngine *engine)
+{
+	HTMLColor *new_color;
+
+	new_color = html_engine_get_document_color (engine);
+
+	if (new_color && !html_color_equal (new_color, engine->insertion_color)) {
+		html_color_unref (engine->insertion_color);
+		engine->insertion_color = new_color;
+		html_color_ref (engine->insertion_color);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 /**
  * html_engine_set_font_style:
  * @engine: An HTMLEngine
@@ -765,22 +879,213 @@ html_engine_update_insertion_font_style (HTMLEngine *engine)
  * from the current one, through an AND mask and an OR mask.
  *
  **/
-void
-html_engine_set_font_style (HTMLEngine *engine,
+struct tmp_font {
+	GtkHTMLFontStyle and_mask;
+	GtkHTMLFontStyle or_mask;
+};
+
+static void
+object_set_font_style (HTMLObject *o, struct tmp_font *tf)
+{
+	if (html_object_is_text (o)) {
+		HTML_TEXT (o)->font_style &= tf->and_mask;
+		HTML_TEXT (o)->font_style |= tf->or_mask;
+	}
+}
+
+gboolean
+html_engine_set_font_style (HTMLEngine *e,
 			    GtkHTMLFontStyle and_mask,
 			    GtkHTMLFontStyle or_mask)
 {
-	g_return_if_fail (engine != NULL);
-	g_return_if_fail (HTML_IS_ENGINE (engine));
-	g_return_if_fail (engine->editable);
+	g_return_val_if_fail (e != NULL, FALSE);
+	g_return_val_if_fail (HTML_IS_ENGINE (e), FALSE);
+	g_return_val_if_fail (e->editable, FALSE);
 
-	if (engine->active_selection) {
-		html_engine_freeze (engine);
-		set_font_style_in_selection (engine, and_mask, or_mask, TRUE);
-		html_engine_thaw (engine);
-		return;
+	if (e->active_selection) {
+		struct tmp_font *tf = g_new (struct tmp_font, 1);
+		tf->and_mask = and_mask;
+		tf->or_mask  = or_mask;
+		html_engine_cut_and_paste (e, "Set font style", (GFunc) object_set_font_style, tf);
+		g_free (tf);
+		return FALSE;
+	} else {
+		GtkHTMLFontStyle old = e->insertion_font_style;
+
+		e->insertion_font_style &= and_mask;
+		e->insertion_font_style |= or_mask;
+
+		return (old == e->insertion_font_style) ? FALSE : TRUE;
+	}
+}
+
+gboolean
+html_engine_toggle_font_style (HTMLEngine *engine, GtkHTMLFontStyle style)
+{
+	GtkHTMLFontStyle cur_style;
+
+	cur_style = html_engine_get_font_style (engine);
+
+	if (cur_style & style)
+		return html_engine_set_font_style (engine, GTK_HTML_FONT_STYLE_MAX & ~style, 0);
+	else
+		return html_engine_set_font_style (engine, GTK_HTML_FONT_STYLE_MAX, style);
+}
+
+static GtkHTMLFontStyle
+inc_dec_size (GtkHTMLFontStyle style, gboolean inc)
+{
+	GtkHTMLFontStyle size;
+
+	if (style == GTK_HTML_FONT_STYLE_DEFAULT)
+		style = GTK_HTML_FONT_STYLE_SIZE_3;
+
+	size = style & GTK_HTML_FONT_STYLE_SIZE_MASK;
+	if (inc && size < GTK_HTML_FONT_STYLE_SIZE_7)
+		size++;
+	else if (!inc && size > GTK_HTML_FONT_STYLE_SIZE_1)
+		size--;
+
+	style &= ~GTK_HTML_FONT_STYLE_SIZE_MASK;
+	style |= size;
+
+	return style;
+}
+
+static void
+inc_dec_size_cb (HTMLObject *obj, gpointer data)
+{
+	if (html_object_is_text (obj))
+		HTML_TEXT (obj)->font_style = inc_dec_size (HTML_TEXT (obj)->font_style, GPOINTER_TO_INT (data));
+}
+
+void
+html_engine_font_size_inc_dec (HTMLEngine *e, gboolean inc)
+{
+	if (e->active_selection)
+		html_engine_cut_and_paste (e, (inc) ? "Increase font size" : "Decrease font size",
+					   (GFunc) inc_dec_size_cb, GINT_TO_POINTER (inc));
+	else
+		e->insertion_font_style = inc_dec_size (e->insertion_font_style, inc);
+}
+
+static void
+set_color (HTMLObject *o, HTMLColor *color)
+{
+	if (html_object_is_text (o))
+		html_text_set_color (HTML_TEXT (o), NULL, color);
+}
+
+void
+html_engine_set_color (HTMLEngine *e, HTMLColor *color)
+{
+	if (e->active_selection) {
+		html_engine_cut_and_paste_begin (e, "Set color");
+		g_list_foreach (e->cut_buffer, (GFunc) set_color, color);
+		html_engine_cut_and_paste_end (e);
+	} else {
+		html_color_unref (e->insertion_color);
+		e->insertion_color = color;
+		html_color_ref (e->insertion_color);
+	}
+}
+
+/* URL/Target
+
+   get actual url/target
+*/
+
+const gchar *
+html_engine_get_url (HTMLEngine *e)
+{
+	return e->insertion_url;
+}
+
+const gchar *
+html_engine_get_target (HTMLEngine *e)
+{
+	return e->insertion_target;
+}
+
+void
+html_engine_set_url (HTMLEngine *e, const gchar *url)
+{
+	if (e->insertion_url)
+		g_free (e->insertion_url);
+	e->insertion_url = g_strdup (url);
+}
+
+void
+html_engine_set_target (HTMLEngine *e, const gchar *target)
+{
+	if (e->insertion_target)
+		g_free (e->insertion_target);
+	e->insertion_target = g_strdup (target);
+}
+
+/* get url/target from document */
+
+static const gchar *
+get_url_or_target_from_selection (HTMLEngine *e, gboolean get_url)
+{
+	const gchar *str = NULL;
+	gboolean backwards;
+	HTMLObject *p;
+
+	g_return_val_if_fail (e->clue != NULL, NULL);
+	g_assert (e->active_selection);
+	g_assert (e->mark != NULL);
+
+	backwards = (e->mark->position < e->cursor->position) ? TRUE : FALSE;
+
+	p = e->cursor->object;
+	while (1) {
+		str = (get_url) ? html_object_get_url (p) : html_object_get_target (p);
+		if (str) break;
+		if (p == e->mark->object) break;
+		p = (backwards) ? html_object_prev_for_cursor (p) : html_object_next_for_cursor (p);
+		g_assert (p != NULL);
 	}
 
-	engine->insertion_font_style &= and_mask;
-	engine->insertion_font_style |= or_mask;
+	return str;
+}
+
+const gchar *
+html_engine_get_document_url (HTMLEngine *e)
+{
+	if (e->active_selection)
+		return get_url_or_target_from_selection (e, TRUE);
+	else
+		return html_object_get_url (e->cursor->object);
+}
+
+const gchar *
+html_engine_get_document_target (HTMLEngine *e)
+{
+	if (e->active_selection)
+		return get_url_or_target_from_selection (e, FALSE);
+	else
+		return html_object_get_target (e->cursor->object);
+}
+
+gboolean
+html_engine_update_insertion_url_and_target (HTMLEngine *engine)
+{
+	const gchar *url, *target;
+	gboolean retval = FALSE;
+
+	url    = html_engine_get_document_url    (engine);
+	target = html_engine_get_document_target (engine);
+
+	if (url != engine->insertion_url) {
+		html_engine_set_url (engine, url);
+		retval = TRUE;
+	}
+
+	if (target != engine->insertion_target) {
+		html_engine_set_target (engine, target);
+		retval = TRUE;
+	}
+
+	return retval;
 }
