@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* htmltype.h
 
-   Copyright (C) 1999 Helix Code, Inc.
+   Copyright (C) 1999, 2000 Helix Code, Inc.
 
    The Gnome Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License as
@@ -33,9 +33,9 @@
 #include "htmlcluev.h"
 #include "htmlembedded.h"
 #include "htmlhidden.h"
+#include "htmliframe.h"
 #include "htmlimage.h"
 #include "htmlimageinput.h"
-#include "htmllinktext.h"
 #include "htmllinktextmaster.h"
 #include "htmlobject.h"
 #include "htmlradio.h"
@@ -73,7 +73,6 @@ html_types_init (void)
 	html_hidden_type_init ();
 	html_image_type_init ();
 	html_imageinput_type_init ();
-	html_link_text_type_init ();
 	html_link_text_master_type_init ();
 	html_object_type_init ();
 	html_radio_type_init ();
@@ -87,6 +86,7 @@ html_types_init (void)
 	html_text_slave_type_init ();
 	html_text_type_init ();
 	html_vspace_type_init ();
+	html_iframe_type_init ();
 
 	types_inited = TRUE;
 }
@@ -155,6 +155,8 @@ html_type_name (HTMLType type)
  		return "TextSlave";
 	case HTML_TYPE_VSPACE:
  		return "VSpace";
+	case HTML_TYPE_IFRAME:
+		return "IFrame";
 	case HTML_TYPE_NONE:
  	case HTML_NUM_TYPES:
 		/* Make compiler happy.  */
@@ -164,3 +166,4 @@ html_type_name (HTMLType type)
 	g_assert_not_reached ();
 	return NULL;
 }
+
